import argparse
import logging
import sys

from ingest_envoy.launchers import IngestCalibrationLauncher
from ingest_envoy.solicitor import Solicitor
from pycapo import CapoConfig

"""
Setup and Launch ingestion via Workspaces 
"""

logger = logging.getLogger("ingest_envoy")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def _get_settings(filename: str) -> dict:
    ingestion_settings = CapoConfig().settings("edu.nrao.workspaces.IngestionSettings")
    staging_root_dir = ingestion_settings.stagingDirectory
    storage_root_dir = ingestion_settings.storageDirectory

    parameters = Solicitor(filename).solicit_parameters()

    parameters["staging_area"] = staging_root_dir
    parameters["storage_area"] = storage_root_dir

    return parameters


def arg_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces Ingestion handler", formatter_class=argparse.RawTextHelpFormatter
    )
    parser.add_argument(
        "--calibration",
        nargs=1,
        action="store",
        required=False,
        help="run ingestion for a calibration product",
    )
    parser.add_argument(
        "--image",
        nargs=1,
        action="store",
        required=False,
        help="run ingestion for an image product (anticipated functionality)",
    )
    return parser


def check_ingest_type(args_type: str, parameters: dict) -> bool:
    if args_type in parameters["workflowName"]:
        return True

    return False


def main():
    args = arg_parser().parse_args()

    if args.calibration is not None:
        arg_type = "calibration"
        parameters = _get_settings(args.calibration[0])

        if check_ingest_type(args_type=arg_type, parameters=parameters):
            ingest_result = IngestCalibrationLauncher(parameters).launch_ingestion()
            if ingest_result == 0:
                # Ingestion succeeded
                logger.info("Ingestion finished successfully!")
            else:
                logger.error("ERROR: Ingestion failure! Please check logs")

        else:
            logger.error(
                f"ERROR: The workflow request to be ingested does not match specified ingestion type {type}."
            )
